@@ -7,7 +7,7 @@ from .blueprint import Blueprint  # noqa
 from .pagination import Page  # noqa
 from .error_handler import ErrorHandlerMixin
 
-__version__ = '0.31.3'
+__version__ = '0.33.0'
 
 
 class Api(APISpecMixin, ErrorHandlerMixin):
@@ -80,11 +80,12 @@ class Api(APISpecMixin, ErrorHandlerMixin):
 
         Must be called after app is initialized.
         """
+        blp_name = blp.name if "name" not in options else options["name"]
 
         self._app.register_blueprint(blp, **options)
 
         # Register views in API documentation for this resource
-        blp.register_views_in_doc(self, self._app, self.spec)
+        blp.register_views_in_doc(self, self._app, self.spec, name=blp_name)
 
         # Add tag relative to this resource to the global tag list
-        self.spec.tag({'name': blp.name, 'description': blp.description})
+        self.spec.tag({'name': blp_name, 'description': blp.description})
