@@ -61,11 +61,11 @@ def _pagination_parameters_schema_factory(
             unknown = ma.EXCLUDE
 
         page = ma.fields.Integer(
-            missing=def_page,
+            load_default=def_page,
             validate=ma.validate.Range(min=1)
         )
         page_size = ma.fields.Integer(
-            missing=def_page_size,
+            load_default=def_page_size,
             validate=ma.validate.Range(min=1, max=def_max_page_size)
         )
 
@@ -124,6 +124,12 @@ class PaginationMetadataSchema(ma.Schema):
         ordered = True
 
 
+PAGINATION_HEADER = {
+    'description': 'Pagination metadata',
+    'schema': PaginationMetadataSchema,
+}
+
+
 class PaginationMixin:
     """Extend Blueprint to add Pagination feature"""
 
@@ -131,7 +137,7 @@ class PaginationMixin:
 
     # Name of field to use for pagination metadata response header
     # Can be overridden. If None, no pagination header is returned.
-    PAGINATION_HEADER_FIELD_NAME = 'X-Pagination'
+    PAGINATION_HEADER_NAME = 'X-Pagination'
 
     # Global default pagination parameters
     # Can be overridden to provide custom defaults
@@ -197,7 +203,7 @@ class PaginationMixin:
                     result = pager(result, page_params=page_params).items
 
                 # Set pagination metadata in response
-                if self.PAGINATION_HEADER_FIELD_NAME is not None:
+                if self.PAGINATION_HEADER_NAME is not None:
                     if page_params.item_count is None:
                         warnings.warn(
                             'item_count not set in endpoint {}.'
@@ -257,7 +263,7 @@ class PaginationMixin:
         """
         if headers is None:
             headers = {}
-        headers[self.PAGINATION_HEADER_FIELD_NAME] = json.dumps(
+        headers[self.PAGINATION_HEADER_NAME] = json.dumps(
             self._make_pagination_metadata(
                 page_params.page,
                 page_params.page_size,
@@ -272,10 +278,9 @@ class PaginationMixin:
         Override this to document custom pagination metadata
         """
         resp_doc['headers'] = {
-            self.PAGINATION_HEADER_FIELD_NAME: {
-                'description': 'Pagination metadata',
-                'schema': PaginationMetadataSchema,
-            }
+            self.PAGINATION_HEADER_NAME:
+            "PAGINATION" if spec.openapi_version.major >= 3
+            else PAGINATION_HEADER
         }
 
     def _prepare_pagination_doc(self, doc, doc_info, *, spec, **kwargs):
