@@ -109,26 +109,33 @@ class TestBlueprint:
                 assert 'consumes' not in get
 
     @pytest.mark.parametrize('openapi_version', ('2.0', '3.0.2'))
-    def test_blueprint_multiple_registrations(self, app, openapi_version):
-        """Check blueprint can be registered multiple times
+    def test_blueprint_register_with_custom_name_and_prefix(
+            self, app, openapi_version
+    ):
+        """Check blueprint can be registered with custom name and prefix
 
-        The internal doc structure is modified during the reigistration
-        process. If it is not deepcopied, the second registration fails.
+        Also check the doc is not modified by the first registration.
         """
         app.config['OPENAPI_VERSION'] = openapi_version
-        blp = Blueprint('test', __name__, url_prefix='/test')
+        blp = Blueprint('test', __name__)
 
         @blp.route('/')
         def func():
             pass
 
         api = Api(app)
-        api.register_blueprint(blp)
-        spec_1 = api.spec.to_dict()
-        api = Api(app)
-        api.register_blueprint(blp)
-        spec_2 = api.spec.to_dict()
-        assert spec_1 == spec_2
+        api.register_blueprint(blp, name="Test 1", url_prefix='/test_1')
+        api.register_blueprint(blp, name="Test 2", url_prefix='/test_2')
+        spec = api.spec.to_dict()
+
+        # Check tags are correctly set and docs only differ by tags
+        assert [t["name"] for t in spec["tags"]] == ["Test 1", "Test 2"]
+        assert list(spec["paths"].keys()) == ["/test_1/", "/test_2/"]
+        path_1 = spec["paths"]["/test_1/"]
+        path_2 = spec["paths"]["/test_2/"]
+        assert path_1["get"].pop("tags") == ["Test 1"]
+        assert path_2["get"].pop("tags") == ["Test 2"]
+        assert path_1 == path_2
 
     @pytest.mark.parametrize('openapi_version', ('2.0', '3.0.2'))
     @pytest.mark.parametrize('location_map', LOCATIONS_MAPPING)
@@ -398,18 +405,38 @@ class TestBlueprint:
         assert http.HTTPStatus(error_code).name in get_responses(api.spec)
 
     @pytest.mark.parametrize('openapi_version', ('2.0', '3.0.2'))
-    def test_blueprint_route_parameters(self, app, openapi_version):
+    @pytest.mark.parametrize('decorate', (True, False))
+    def test_blueprint_route_add_url_rule_parameters(
+        self, app, openapi_version, decorate
+    ):
         """Check path parameters docs are merged with auto docs"""
         app.config['OPENAPI_VERSION'] = openapi_version
         api = Api(app)
         blp = Blueprint('test', __name__, url_prefix='/test')
 
-        @blp.route('/<int:item_id>', parameters=[
-            'TestParameter',
-            {'name': 'item_id', 'in': 'path', 'description': 'Item ID'},
-        ])
-        def get(item_id):
-            pass
+        if decorate:
+            @blp.route('/<int:item_id>', parameters=[
+                'TestParameter',
+                {'name': 'item_id', 'in': 'path', 'description': 'Item ID'},
+            ])
+            def get(item_id):
+                pass
+        else:
+            def get(item_id):
+                pass
+
+            blp.add_url_rule(
+                '/<int:item_id>',
+                view_func=get,
+                parameters=[
+                    'TestParameter',
+                    {
+                        'name': 'item_id',
+                        'in': 'path',
+                        'description': 'Item ID'
+                    },
+                ]
+            )
 
         api.register_blueprint(blp)
         spec = api.spec.to_dict()
@@ -1126,6 +1153,37 @@ class TestBlueprint:
         assert 'get' in paths['/test/route_1']
         assert 'get' in paths['/test/route_2']
 
+    @pytest.mark.parametrize('as_method_view', (True, False))
+    @pytest.mark.parametrize('endpoint', (None, "test"))
+    def test_blueprint_add_url_rule(self, app, as_method_view, endpoint):
+        api = Api(app)
+        blp = Blueprint('test', __name__, url_prefix='/test')
+
+        if as_method_view:
+            class Resource(MethodView):
+
+                def get(self):
+                    pass
+
+            blp.add_url_rule('/test', endpoint=endpoint, view_func=Resource)
+
+        else:
+            def func():
+                pass
+
+            blp.add_url_rule('/test', endpoint=endpoint, view_func=func)
+
+        api.register_blueprint(blp)
+        paths = api.spec.to_dict()['paths']
+
+        assert 'get' in paths['/test/test']
+        assert 'get' in paths['/test/test']
+
+    def test_blueprint_add_url_rule_without_view_func(self):
+        blp = Blueprint('test', __name__, url_prefix='/test')
+        with pytest.raises(TypeError, match="view_func must be provided"):
+            blp.add_url_rule('/test', 'dummy_endpoint')
+
     def test_blueprint_response_tuple(self, app):
         api = Api(app)
         blp = Blueprint('test', __name__, url_prefix='/test')
@@ -1332,18 +1390,14 @@ class TestBlueprint:
         operation = api.spec.to_dict()['paths']['/test/'][method.lower()]
         responses = operation.get('responses', {})
         response_headers = responses.get("204", {}).get("headers", {})
-        parameters = operation.get('parameters', {})
-
-        def find_header(name):
-            return [x for x in parameters
-                    if x.get('in') == 'header' and x.get('name') == name]
+        parameters = operation.get('parameters', [])
 
         if not decorate or etag_disabled:
             assert '304' not in responses
             assert '412' not in responses
             assert '428' not in responses
-            assert len(find_header('If-None-Match')) == 0
-            assert len(find_header('If-Match')) == 0
+            assert "IF_NONE_MATCH" not in parameters
+            assert "IF_MATCH" not in parameters
             assert 'ETag' not in response_headers
         else:
             assert ('304' in responses) == (method in ['GET', 'HEAD'])
@@ -1351,9 +1405,13 @@ class TestBlueprint:
                 method in ['PUT', 'PATCH', 'DELETE'])
             assert ('428' in responses) == (
                 method in ['PUT', 'PATCH', 'DELETE'])
-            assert (len(find_header('If-None-Match')) == 1) == (
-                method in ['GET', 'HEAD'])
-            assert (len(find_header('If-Match')) == 1) == (
-                method in ['PUT', 'PATCH', 'DELETE'])
-            assert not has_response or ('ETag' in response_headers) == (
-                method in ['GET', 'HEAD', 'POST', 'PUT', 'PATCH'])
+            assert (
+                build_ref(api.spec, 'parameter', 'IF_NONE_MATCH') in parameters
+            ) == (method in ['GET', 'HEAD'])
+            assert (
+                build_ref(api.spec, 'parameter', 'IF_MATCH') in parameters
+            ) == (method in ['PUT', 'PATCH', 'DELETE'])
+            assert not has_response or (
+                response_headers.get("ETag") ==
+                build_ref(api.spec, 'header', "ETAG")
+            ) == (method in ['GET', 'HEAD', 'POST', 'PUT', 'PATCH'])
